require 'thor'

module Remenv
  class CLI < Thor
    class_option :adapter,   default: 'redis'
    class_option :stage,     default: 'development'
    class_option :namespace, default: 'remenv'
    class_option :url

    no_commands do
      def merged_settings
        defaults = Remenv::Config.load(stage: options[:stage])
        symbolized = defaults.each_with_object({}) do |(key, val), hash|
          hash[key.to_sym] = val
        end

        symbolized.merge!(options)
      end

      def configure!
        settings = merged_settings

        if settings['adapter'] == 'redis'
          require 'remenv/adapter/redis'

          Remenv.adapter = Remenv::Adapter::Redis.new(
            url:  settings['url'],
            namespace: settings['namespace']
          )
        end
      end

      def with_configuration
        configure!
        yield
      end
    end

    desc 'show [KEY]', 'Show value for KEY or all stored key/value pairs'
    def show(key = nil)
      with_configuration do
        pairs = key.nil? ? Remenv.show : Remenv.get(key)

        puts Remenv::Formatter.pretty(pairs)
      end
    end

    desc 'clear', 'Clear all stored key/value pairs'
    def clear
      with_configuration do
        Remenv.clear!

        if yes?('Are you sure? (y/n)')
          say 'OK! Cleared', :green
        else
          say 'Aborted', :red
        end
      end
    end

    desc 'set KEY=VALUE', 'Store VALUE for KEY'
    def set(*pairs)
      with_configuration do
        hash = pairs.each_with_object({}) do |pair, memo|
          key, value = pair.split('=')
          memo[key] = value
        end

        Remenv.set(hash)

        say 'OK!', :green
        puts Remenv::Formatter.pretty(hash)
      end
    end

    desc 'unset KEY', 'Remove values for KEY'
    def unset(*keys)
      with_configuration do
        Remenv.unset(*keys)

        say 'OK!', :green
      end
    end

    desc 'dump FILENAME', 'Dump all stored key/value pairs to FILENAME'
    def dump(filename)
      with_configuration do
        Remenv.dump(filename)

        say "OK! Dumped to #{filename}", :green
      end
    end

    desc 'load FILENAME', 'Load all key/value pairs from FILENAME'
    def load(filename)
      with_configuration do
        if yes?('This will clear all existing variables: (y/n)')
          Remenv.load(filename)
          say 'OK! Loaded', :green
        else
          say 'Aborted', :red
        end
      end
    end
  end
end
